﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Pacman02_UglyOOP
{
    class Program
    {
         static string[] mapS = {
            "##############",
            "#..#.###@...##",
            "##.#...#.##..#",
            "##.#.###.#####",
            "##¤..........#",
            "##############"
                              };

        class Vec2
        {
            public int x = 0;
            public int y = 0;

            public Vec2(){}
            public Vec2(int x, int y) { this.x = x; this.y = y; }
            public Vec2 add(Vec2 other) { return new Vec2(x+other.x, y+other.y);}
            public void setNull() { x = y = 0; }
            public bool isNull() { return x == 0 && y == 0; }
        }
        
        class GameObject
        {
            public enum GO_Type
            {
                WALL,
                HERO,
                ENEMY,
                COIN
            }
            
            public void draw()
            {
                System.Console.SetCursorPosition(position.x, position.y + 1);
                if (mType == GO_Type.WALL)
                    System.Console.Write('#');
                if (mType == GO_Type.COIN)
                    System.Console.Write('.');
                if (mType == GO_Type.ENEMY)
                    System.Console.Write('@');
                if (mType == GO_Type.HERO)
                    System.Console.Write('¤');
            }

            public void update()
            {
                if (mType == GO_Type.HERO)
                {
                    if (System.Console.KeyAvailable)
                    {
                        ConsoleKeyInfo keyinfo = System.Console.ReadKey(true);
                        move.setNull();
                        if (keyinfo.Key == ConsoleKey.LeftArrow)
                            move.x = -1;
                        if (keyinfo.Key == ConsoleKey.RightArrow)
                            move.x = 1;
                        if (keyinfo.Key == ConsoleKey.UpArrow)
                            move.y = -1;
                        if (keyinfo.Key == ConsoleKey.DownArrow)
                            move.y = 1;
                        if (keyinfo.Key == ConsoleKey.Escape)
                            System.Environment.Exit(0);                        
                    }
                }
            }

            public void allowMove()
            {
                System.Console.SetCursorPosition(position.x, position.y + 1);
                System.Console.Write(" ");

                position = position.add(move);
                move.setNull();

                draw();
            }

            public void cancelMove()
            {
                move.setNull();
            }

            public GO_Type mType;
            public Vec2 position = new Vec2();
            public Vec2 move = new Vec2();
            public int score = 0;
        }

        
        class Game
        {
            GameObject[][] map;
            GameObject player = null;
                
            public void run()
            {
                int cols = mapS[0].Length;
                int rows = mapS.Length;
                map = new GameObject[rows][];
                for (int j = 0; j < rows; ++j)
                {
                    map[j] = new GameObject[cols];
                    for (int i = 0; i < cols; ++i)
                    {
                        GameObject go = new GameObject();
                        map[j][i] = go;
                        go.position = new Vec2(i, j);
                        if (mapS[j][i] == '#')
                            go.mType = GameObject.GO_Type.WALL;
                        else if (mapS[j][i] == '.')
                            go.mType = GameObject.GO_Type.COIN;
                        else if (mapS[j][i] == '¤')
                        {
                            go.mType = GameObject.GO_Type.HERO;
                            player = go;
                        }
                        else if (mapS[j][i] == '@')
                            go.mType = GameObject.GO_Type.ENEMY;
                    }
                }

                //draw level
                for (int i = 0; i < cols; ++i)
                {
                    for (int j = 0; j < rows; ++j)
                    {
                        map[j][i].draw();
                    }
                }
                System.Console.SetCursorPosition(0, 0);
                System.Console.Write("Score: 0");

                while (true)
                {
                    for (int i = 0; i < cols; ++i)
                    {
                        for (int j = 0; j < rows; ++j)
                        {
                            if(map[j][i] != null)
                                map[j][i].update();
                            if(map[j][i] == player)
                            {
                                if (!player.move.isNull())
                                {
                                    Vec2 nextPosition = player.position.add(player.move);
                                    GameObject obj = map[nextPosition.y][nextPosition.x];
                                    if (obj.mType != GameObject.GO_Type.WALL) //not a wall hit
                                    {
                                        if (obj.mType == GameObject.GO_Type.COIN)
                                        {
                                            player.score++;
                                            map[nextPosition.y][nextPosition.x] = null;
                                            drawGUI();
                                        }
                                        else if (obj.mType == GameObject.GO_Type.ENEMY)
                                        {
                                            player = null;
                                            drawGUI();
                                        }
                                        player.allowMove();
                                        map[player.position.y][player.position.x] = player;
                                    }
                                    else
                                        player.cancelMove();
                                }
                            }
                        }
                    }
                }
            }

            void drawGUI()
            {
                if(player == null)
                {
                    System.Console.SetCursorPosition(0, 0);
                    System.Console.Write("GAME OVER");
                    System.Threading.Thread.Sleep(3000);
                    System.Environment.Exit(0);
                }
                System.Console.SetCursorPosition(0, 0);
                System.Console.Write("Score: " + player.score);
            }
        }
        static void Main(string[] args)
        {
            Game game = new Game();
            game.run();
        }
    }
}